import {Component, OnDestroy, OnInit, ViewChild} from "@angular/core";
import {StakeholderService} from "../services/stakeholder.service";
import {EnvProperties} from "../openaireLibrary/utils/properties/env-properties";
import {EnvironmentSpecificService} from "../openaireLibrary/utils/properties/environment-specific.service";
import {Stakeholder} from "../utils/entities/stakeholder";
import {Subscriber, zip} from "rxjs";
import {StakeholderUtils} from "../utils/indicator-utils";
import {FormBuilder, FormGroup, Validators} from "@angular/forms";
import {AlertModal} from "../openaireLibrary/utils/modal/alert";
import {StakeholderCreator} from "../utils/entities/stakeholderCreator";
import {Option} from "../openaireLibrary/dashboard/sharedComponents/input/input.component";

@Component({
  selector: 'home',
  templateUrl: "./manageStakeholders.component.html"
})
export class ManageStakeholdersComponent implements OnInit, OnDestroy {
  
  public properties: EnvProperties;
  public loading: boolean = true;
  public stakeholderUtils: StakeholderUtils = new StakeholderUtils();
  public defaultStakeholders: Stakeholder[];
  public stakeholders: Stakeholder[];
  public stakeholder: Stakeholder;
  public stakeholderFb: FormGroup;
  public index: number;
  
  /**
   *  Filtered Stakeholders
   */
  public displayDefaultStakeholders: Stakeholder[];
  public displayStakeholders: Stakeholder[];
  
  /**
   * Top filters
   */
  public filters: FormGroup;
  public all: Option = {
    value: 'all',
    label: 'All'
  };
  
  /**
   * Grid or List View
   */
  public grid: boolean = true;
  private subscriptions: any[] = [];
  
  @ViewChild('editStakeholderModal') editStakeholderModal: AlertModal;
  @ViewChild('deleteStakeholderModal') deleteStakeholderModal: AlertModal;
  
  constructor(private stakeholderService: StakeholderService,
              private propertiesService: EnvironmentSpecificService,
              private fb: FormBuilder) {
  }
  
  ngOnInit(): void {
    this.buildFilters();
    this.propertiesService.loadEnvironment()
      .then(properties => {
        this.properties = properties;
        let data = zip(
          this.stakeholderService.getDefaultStakeholders(this.properties.monitorServiceAPIURL),
          this.stakeholderService.getStakeholders(this.properties.monitorServiceAPIURL)
        );
        data.subscribe(res => {
          this.defaultStakeholders = res[0];
          this.stakeholders = res[1];
          this.displayDefaultStakeholders = res[0];
          this.displayStakeholders = res[1];
          this.loading = false;
        });
      });
  }
  
  ngOnDestroy(): void {
    this.subscriptions.forEach(value => {
      if (value instanceof Subscriber) {
        value.unsubscribe();
      }
    });
  }
  
  public changeGrid(value) {
    this.grid = value;
  }
  
  private buildFilters() {
    this.filters = this.fb.group({
      privacy: this.fb.control('all'),
      status: this.fb.control('all'),
      keyword: this.fb.control('')
    });
    this.subscriptions.push(this.filters.get('privacy').valueChanges.subscribe(value => {
      this.onPrivacyChange(value);
    }));
    this.subscriptions.push(this.filters.get('status').valueChanges.subscribe(value => {
      this.onStatusChange(value);
    }));
    this.subscriptions.push(this.filters.get('keyword').valueChanges.subscribe(value => {
      this.onKeywordChange(value);
    }));
  }
  
  onPrivacyChange(value) {
    this.displayDefaultStakeholders = this.filterPrivacy(this.defaultStakeholders, value);
    this.displayStakeholders = this.filterPrivacy(this.stakeholders, value);
  }
  
  onStatusChange(value) {
    this.displayDefaultStakeholders = this.filterStatus(this.defaultStakeholders, value);
    this.displayStakeholders = this.filterStatus(this.stakeholders, value);
  }
  
  onKeywordChange(value) {
    this.displayDefaultStakeholders = this.filterByKeyword(this.defaultStakeholders, value);
    this.displayStakeholders = this.filterByKeyword(this.stakeholders, value);
  }
  
  
  private filterPrivacy(stakeholders: Stakeholder[], value): Stakeholder[] {
    if (value === 'all') {
      return stakeholders;
    } else {
      return stakeholders.filter(stakeholder => stakeholder.isPublic === value);
    }
  }
  
  private filterStatus(stakeholders: Stakeholder[], value): Stakeholder[] {
    if (value === 'all') {
      return stakeholders;
    } else {
      return stakeholders.filter(stakeholder => stakeholder.isActive === value);
    }
  }
  
  private filterByKeyword(stakeholders: Stakeholder[], value): Stakeholder[] {
    if (value === null || value === '') {
      return stakeholders;
    } else {
      return stakeholders.filter(stakeholder => (
        stakeholder.index_id && stakeholder.index_id.toLowerCase().includes(value.toLowerCase())) ||
        stakeholder.index_shortName && stakeholder.index_shortName.toLowerCase().includes(value.toLowerCase()) ||
        stakeholder.index_name && stakeholder.index_name.toLowerCase().includes(value.toLowerCase())
      );
    }
  }
  
  public editStakeholder(stakeholder: Stakeholder = null, isDefault: boolean = false) {
    if (isDefault) {
      this.index = (stakeholder) ? this.defaultStakeholders.findIndex(value => value._id === stakeholder._id) : -1;
    } else {
      this.index = (stakeholder) ? this.stakeholders.findIndex(value => value._id === stakeholder._id) : -1;
    }
    if (!stakeholder) {
      this.stakeholder = new Stakeholder(null, null, null,
        null, null, null, false, false, null);
    } else {
      this.stakeholder = stakeholder;
    }
    this.stakeholderFb = this.fb.group({
      _id: this.fb.control(this.stakeholder._id),
      index_name: this.fb.control(this.stakeholder.index_name, Validators.required),
      index_id: this.fb.control(this.stakeholder.index_id, Validators.required),
      index_shortName: this.fb.control(this.stakeholder.index_shortName, Validators.required),
      defaultId: this.fb.control((isDefault) ? null : 'id'),
      creationDate: this.fb.control(this.stakeholder.creationDate),
      alias: this.fb.control(this.stakeholder.alias,
        [
          Validators.required,
          this.stakeholderUtils.aliasValidator(
            (isDefault) ?
              this.defaultStakeholders.filter(stakeholder => stakeholder.alias !== this.stakeholder.alias) :
              this.stakeholders.filter(stakeholder => stakeholder.alias !== this.stakeholder.alias)
          )]
      ),
      isDefault: this.fb.control(isDefault),
      isPublic: this.fb.control(this.stakeholder.isPublic),
      isActive: this.fb.control(this.stakeholder.isActive),
      type: this.fb.control(this.stakeholder.type, Validators.required),
      topics: this.fb.control(this.stakeholder.topics),
      managers: this.fb.control(this.stakeholder.managers),
      logoUrl: this.fb.control(this.stakeholder.logoUrl)
    });
    if (this.index !== -1) {
      if (this.stakeholderFb.value.type) {
        setTimeout(() => {
          this.stakeholderFb.get('type').disable();
        }, 0);
      } else {
        setTimeout(() => {
          this.stakeholderFb.get('type').enable();
        }, 0);
      }
      this.editStakeholderModal.okButtonText = 'Save Changes';
    } else {
      setTimeout(() => {
        this.stakeholderFb.get('type').enable();
      }, 0);
      this.editStakeholderModal.okButtonText = 'Create';
    }
    this.editStakeholderModal.cancelButtonText = 'Cancel';
    this.editStakeholderModal.okButtonLeft = false;
    this.editStakeholderModal.alertMessage = false;
    this.editStakeholderModal.open();
  }
  
  public saveStakeholder() {
    if (this.index === -1) {
      if (!this.stakeholderFb.value.isDefault) {
        let stakeholder = this.defaultStakeholders.find(value => value.type === this.stakeholderFb.value.type);
        this.stakeholderFb.get('defaultId').setValue(stakeholder._id);
        this.stakeholderFb.setValue(this.stakeholderUtils.createFunderFromDefaultProfile(this.stakeholderFb.value,
          stakeholder.topics));
       /* this.stakeholderFb.setValue(this.stakeholderUtils.createFunderFromDefaultProfile(this.stakeholderFb.value,
          StakeholderCreator.createFunderDefaultProfile().topics));*/
      } else {
        this.stakeholderFb.setValue(StakeholderCreator.createFunderDefaultProfile());
      }
      this.stakeholderService.buildStakeholder(this.properties.monitorServiceAPIURL, this.stakeholderFb.value).subscribe(stakeholder => {
        if (stakeholder.defaultId === null) {
          this.defaultStakeholders.push(stakeholder);
        } else {
          this.stakeholders.push(stakeholder);
        }
      });
    } else {
      this.stakeholderFb.get('type').enable();
      this.stakeholderService.saveElement(this.properties.monitorServiceAPIURL, this.stakeholderFb.value).subscribe(stakeholder => {
        if (stakeholder.isDefaultProfile) {
          this.defaultStakeholders[this.index] = stakeholder;
        } else {
          this.stakeholders[this.index] = stakeholder;
        }
      });
    }
  }
  
  public deleteStakeholderOpen(stakeholder: Stakeholder) {
    this.stakeholder = stakeholder;
    this.deleteStakeholderModal.alertTitle = 'Delete ' + this.stakeholder.index_name;
    this.deleteStakeholderModal.cancelButtonText = 'No';
    this.deleteStakeholderModal.okButtonText = 'Yes';
    this.deleteStakeholderModal.message = 'This stakeholder will permanently be deleted. Are you sure you want to proceed?';
    this.deleteStakeholderModal.open();
  }
  
  public deleteStakeholder() {
    if (!this.stakeholder.defaultId) {
      this.index = (this.stakeholder) ? this.defaultStakeholders.findIndex(value => value._id === this.stakeholder._id) : -1;
    } else {
      this.index = (this.stakeholder) ? this.stakeholders.findIndex(value => value._id === this.stakeholder._id) : -1;
    }
    this.stakeholderService.deleteElement(this.properties.monitorServiceAPIURL, [this.stakeholder._id]).subscribe(() => {
      if (!this.stakeholder.defaultId) {
        this.defaultStakeholders.splice(this.index, 1);
      } else {
        this.stakeholders.splice(this.index, 1);
      }
    });
  }
}
