import {AfterViewInit, Component, Input, OnChanges, OnDestroy, OnInit, SimpleChanges, ViewChild} from "@angular/core";
import {Indicator, IndicatorPath, IndicatorType, Section, Stakeholder} from "../utils/entities/stakeholder";
import {IndicatorUtils} from "../utils/indicator-utils";
import {FormArray, FormBuilder, FormControl, FormGroup, Validators} from "@angular/forms";
import {AlertModal} from "../openaireLibrary/utils/modal/alert";
import {StatisticsService} from "../utils/services/statistics.service";
import {HelperFunctions} from "../openaireLibrary/utils/HelperFunctions.class";
import {DomSanitizer} from "@angular/platform-browser";
import {StakeholderService} from "../services/stakeholder.service";
import {EnvProperties} from "../openaireLibrary/utils/properties/env-properties";
import {Subscriber} from "rxjs";
import {LayoutService} from "../openaireLibrary/dashboard/sharedComponents/sidebar/layout.service";
import {Option} from "../openaireLibrary/dashboard/sharedComponents/input/input.component";

declare var UIkit;

@Component({
  selector: 'indicators',
  templateUrl: './indicators.component.html'
})
export class IndicatorsComponent implements OnInit, OnDestroy, OnChanges, AfterViewInit {
  
  @Input()
  public properties: EnvProperties = null;
  @Input()
  public topicIndex: number = 0;
  @Input()
  public categoryIndex: number = 0;
  @Input()
  public subcategoryIndex: number = 0;
  @Input()
  public stakeholder: Stakeholder = null;
  public indicatorUtils: IndicatorUtils = new IndicatorUtils();
  public indicatorFb: FormGroup;
  /**
   * Editable indicator
   */
  public section: Section;
  public indicator: Indicator;
  public index: number = -1;
  /**
   * Displayed chart and numbers base on Top filters
   */
  public displayCharts: Section[] = [];
  public displayNumbers: Section[] = [];
  /**
   * Top filters
   */
  public filters: FormGroup;
  public all: Option = {
    value: 'all',
    label: 'All'
  };
  /**
   * Toggles
   */
  public grid: boolean = true;
  private subscriptions: any[] = [];
  private urlSubscriptions: any[] = [];
  @ViewChild('editModal') editModal: AlertModal;
  @ViewChild('deleteModal') deleteModal: AlertModal;
  
  constructor(private layoutService: LayoutService,
              private stakeholderService: StakeholderService,
              private statisticsService: StatisticsService,
              private fb: FormBuilder,
              private sanitizer: DomSanitizer) {
  }
  
  ngOnInit(): void {
    this.buildFilters();
  }
  
  ngOnDestroy(): void {
    this.subscriptions.forEach(value => {
      if (value instanceof Subscriber) {
        value.unsubscribe();
      } else if (value instanceof Function) {
        value();
      }
    });
  }
  
  ngAfterViewInit(): void {
    if (document !== undefined) {
      let callback = (list, type: IndicatorType): void => {
        let items: HTMLCollection = list.current.children;
        let reordered = [];
        for (let i = 0; i < items.length; i++) {
          if (items.item(i).id) {
            reordered.push(items.item(i).id);
          }
        }
        this.reorderIndicators(list.current.id.toString().split('-')[1], type, reordered);
      };
      this.numbers.forEach((section) => {
        this.subscriptions.push(UIkit.util.on(document, 'moved', '#chart' + section._id, (list): void => {
          callback(list, "number");
        }));
        this.subscriptions.push(UIkit.util.on(document, 'added', '#chart' + section._id, (list): void => {
          callback(list, "number");
        }));
        this.subscriptions.push(UIkit.util.on(document, 'removed', '#chart' + section._id, (list): void => {
          callback(list, "number");
        }));
      });
      this.charts.forEach((section) => {
        this.subscriptions.push(UIkit.util.on(document, 'moved', '#chart-' + section._id, (list): void => {
          callback(list, "chart");
        }));
        this.subscriptions.push(UIkit.util.on(document, 'added', '#chart-' + section._id, (list): void => {
          callback(list, "chart");
        }));
        this.subscriptions.push(UIkit.util.on(document, 'removed', '#chart-' + section._id, (list): void => {
          callback(list, "chart");
        }));
      });
    }
  }
  
  ngOnChanges(changes: SimpleChanges): void {
    if (this.canEdit) {
      if (changes.topicIndex || changes.categoryIndex || changes.subcategoryIndex) {
        this.buildFilters();
      }
      this.filterCharts();
      this.filterNumbers();
    }
  }
  
  public toggleOpen(event = null) {
    if (!event) {
      this.layoutService.setOpen(!this.open);
    } else if (event && event['value'] === true) {
      this.layoutService.setOpen(false);
    }
  }
  
  public changeGrid(value) {
    this.grid = value;
  }
  
  private buildFilters() {
    this.filters = this.fb.group({
      chartType: this.fb.control('all'),
      privacy: this.fb.control('all'),
      status: this.fb.control('all'),
      keyword: this.fb.control('')
    });
    this.subscriptions.push(this.filters.get('chartType').valueChanges.subscribe(value => {
      this.onChartTypeChange(value);
    }));
    this.subscriptions.push(this.filters.get('privacy').valueChanges.subscribe(value => {
      this.onPrivacyChange(value);
    }));
    this.subscriptions.push(this.filters.get('status').valueChanges.subscribe(value => {
      this.onStatusChange(value);
    }));
    this.subscriptions.push(this.filters.get('keyword').valueChanges.subscribe(value => {
      this.onKeywordChange(value);
    }));
  }
  
  filterCharts() {
    this.displayCharts = this.filterChartType(this.filterPrivacy(
      this.filterStatus(this.filterByKeyword(this.charts, this.filters.value.keyword),
        this.filters.value.status),
      this.filters.value.privacy),
      this.filters.value.chartType
    );
  }
  
  filterNumbers() {
    this.displayNumbers = this.filterPrivacy(this.filterStatus(
      this.filterByKeyword(this.numbers, this.filters.value.keyword),
      this.filters.value.status),
      this.filters.value.privacy);
  }
  
  onChartTypeChange(value) {
    this.displayCharts = this.filterChartType(this.charts, value);
  }
  
  onPrivacyChange(value) {
    this.displayCharts = this.filterPrivacy(this.charts, value);
    this.displayNumbers = this.filterPrivacy(this.numbers, value);
  }
  
  onStatusChange(value) {
    this.displayCharts = this.filterStatus(this.charts, value);
    this.displayNumbers = this.filterStatus(this.numbers, value);
  }
  
  onKeywordChange(value) {
    this.displayCharts = this.filterByKeyword(this.charts, value);
    this.displayNumbers = this.filterByKeyword(this.numbers, value);
  }
  
  private filterChartType(sections: Section[], value): Section[] {
    if (value === 'all') {
      return sections;
    } else {
      return sections.filter(section =>
        section.indicators = section.indicators.filter(indicator =>
          indicator.indicatorPaths.filter(indicatorPath => indicatorPath.type === value).length > 0));
    }
  }
  
  private filterPrivacy(sections: Section[], value): Section[] {
    if (value === 'all') {
      return sections;
    } else {
      return sections.filter(section =>
        section.indicators = section.indicators.filter(indicator => indicator.isPublic === value));
    }
  }
  
  private filterStatus(sections: Section[], value): Section[] {
    if (value === 'all') {
      return sections;
    } else {
      return sections.filter(section =>
        section.indicators = section.indicators.filter(indicator => indicator.isActive === value));
    }
  }
  
  private filterByKeyword(sections: Section[], value): Section[] {
    if (value === null || value === '') {
      return sections;
    } else {
      return sections.filter(section =>
        section.indicators = section.indicators.filter(indicator => (indicator.name && indicator.name.toLowerCase().includes(value.toLowerCase()))
          || (indicator.description && indicator.description.toLowerCase().includes(value.toLowerCase()))));
    }
  }
  
  get charts(): Section[] {
    return this.stakeholder.topics[this.topicIndex].categories[this.categoryIndex].subCategories[this.subcategoryIndex].charts;
  }
  
  set charts(sections: Section[]) {
    this.stakeholder.topics[this.topicIndex].categories[this.categoryIndex].subCategories[this.subcategoryIndex].charts = sections;
  }
  
  get numbers(): Section[] {
    return this.stakeholder.topics[this.topicIndex].categories[this.categoryIndex].subCategories[this.subcategoryIndex].numbers;
  }
  
  set numbers(sections: Section[]) {
    this.stakeholder.topics[this.topicIndex].categories[this.categoryIndex].subCategories[this.subcategoryIndex].numbers = sections;
  }
  
  get open(): boolean {
    return this.layoutService.open;
  }
  
  get canReorder(): boolean {
    return this.filters.value.chartType === 'all' && this.filters.value.privacy === 'all' &&
      this.filters.value.status === 'all' && this.filters.value.keyword === '' && this.grid;
  }
  
  get canEdit() {
    return this.stakeholder &&
      this.stakeholder.topics[this.topicIndex] &&
      this.stakeholder.topics[this.topicIndex].categories[this.categoryIndex] &&
      this.stakeholder.topics[this.topicIndex].categories[this.categoryIndex].subCategories[this.subcategoryIndex];
  }
  
  public get indicatorPaths(): FormArray {
    return this.indicatorFb.get('indicatorPaths') as FormArray;
  }
  
  public getParameters(index: number): FormArray {
    return this.indicatorPaths.at(index).get('parameters') as FormArray;
  }
  
  public getParameter(index: number, key: string): FormControl {
    return this.getParameters(index).controls.filter(control => control.value.key === key)[0] as FormControl;
  }
  
  private getSecureUrlByStakeHolder(indicatorPath: IndicatorPath) {
    return this.sanitizer.bypassSecurityTrustResourceUrl(
      this.statisticsService.getChartUrl(indicatorPath.source, this.indicatorUtils.getFullUrl(indicatorPath)));
  }
  
  private getUrlByStakeHolder(indicatorPath: IndicatorPath) {
    return this.statisticsService.getChartUrl(indicatorPath.source, this.indicatorUtils.getFullUrl(indicatorPath));
  }
  
  public addIndicatorPath(value: string = '', parameters: FormArray = new FormArray([])) {
    this.indicatorPaths.push(this.fb.group({
        url: this.fb.control(value, [Validators.required,
          Validators.pattern('https?:\/\/(?:www\.|(?!www))[a-zA-Z0-9][a-zA-Z0-9-]+[a-zA-Z0-9]\.[^\s]{2,}|www\.' +
            '[a-zA-Z0-9][a-zA-Z0-9-]+[a-zA-Z0-9]\.[^\s]{2,}|https?:\/\/(?:www\.|(?!www))[a-zA-Z0-9]+\.[^\s]{2,}|www\.' +
            '[a-zA-Z0-9]+\.[^\s]{2,}')]),
        parameters: parameters
      }
    ));
    let index = this.indicatorPaths.length - 1;
    this.urlSubscriptions.push(this.indicatorPaths.at(index).get('url').valueChanges.subscribe(value => {
        if (this.indicatorPaths.at(index).get('url').valid) {
          let indicatorPath: IndicatorPath = this.indicatorUtils.generateIndicatorByChartUrl(this.statisticsService.getChartSource(value), value, 'bar', this.stakeholder);
          let parameters = this.getParametersAsFormArray(indicatorPath);
          (this.indicatorPaths.at(index) as FormGroup).setControl('parameters', parameters);
          if (!this.indicator.indicatorPaths[index]) {
            this.indicator.indicatorPaths[index] = indicatorPath;
            this.indicator.indicatorPaths[index].safeResourceUrl = this.getSecureUrlByStakeHolder(indicatorPath);
          } else {
            indicatorPath.safeResourceUrl = this.indicator.indicatorPaths[index].safeResourceUrl;
            this.indicator.indicatorPaths[index] = indicatorPath;
          }
        }
      })
    );
  }
  
  private getParametersAsFormArray(indicatorPath: IndicatorPath): FormArray {
    let parameters = this.fb.array([]);
    if (indicatorPath.parameters) {
      Object.keys(indicatorPath.parameters).forEach(key => {
        if (this.indicatorUtils.ignoredParameters.indexOf(key) === -1) {
          if (this.indicatorUtils.parametersValidators.has(key)) {
            parameters.push(this.fb.group({
              key: this.fb.control(key),
              value: this.fb.control(indicatorPath.parameters[key], this.indicatorUtils.parametersValidators.get(key))
            }));
          } else {
            parameters.push(this.fb.group({
              key: this.fb.control(key),
              value: this.fb.control(indicatorPath.parameters[key])
            }));
          }
        }
      });
    }
    return parameters;
  }
  
  public editChartIndicatorOpen(section: Section, id = null) {
    this.urlSubscriptions.forEach(value => {
      if (value instanceof Subscriber) {
        value.unsubscribe();
      }
    });
    this.section = section;
    this.index = (id) ? section.indicators.findIndex(value => value._id === id) : -1;
    if (this.index !== -1) {
      this.indicator = HelperFunctions.copy(this.section.indicators[this.index]);
      this.indicatorFb = this.fb.group({
        id: this.fb.control(this.indicator._id),
        name: this.fb.control(this.indicator.name/*, Validators.required*/),
        description: this.fb.control(this.indicator.description),
        isPublic: this.fb.control(this.indicator.isPublic),
        isActive: this.fb.control(this.indicator.isActive),
        indicatorPaths: this.fb.array([]),
        width: this.fb.control(this.indicator.width),
      });
      this.indicator.indicatorPaths.forEach(indicatorPath => {
        this.addIndicatorPath(this.getUrlByStakeHolder(indicatorPath), this.getParametersAsFormArray(indicatorPath));
        indicatorPath.safeResourceUrl = this.getSecureUrlByStakeHolder(indicatorPath)
      });
    } else {
      this.indicator = new Indicator('', '', 'chart', 'small', false, false, []);
      this.indicatorFb = this.fb.group({
        id: this.fb.control(null),
        name: this.fb.control(''),
        description: this.fb.control(''),
        isPublic: this.fb.control(false),
        isActive: this.fb.control(false),
        indicatorPaths: this.fb.array([]),
        width: this.fb.control('small', Validators.required),
      });
      this.addIndicatorPath();
    }
    this.editModal.cancelButtonText = 'Cancel';
    this.editModal.okButtonLeft = false;
    this.editModal.alertMessage = false;
    if (this.index === -1) {
      this.editModal.alertTitle = 'Create a new chart indicator';
      this.editModal.okButtonText = 'Save';
    } else {
      this.editModal.alertTitle = 'Edit chart indicator\'s information';
      this.editModal.okButtonText = 'Save Changes';
    }
    this.editModal.open();
  }
  
  saveIndicator() {
    if (this.indicator.type === 'chart') {
      this.indicator = this.indicatorUtils.generateIndicatorByForm(this.indicatorFb.value, this.indicator.indicatorPaths);
    }
    let path = [
      this.stakeholder._id,
      this.stakeholder.topics[this.topicIndex]._id,
      this.stakeholder.topics[this.topicIndex].categories[this.categoryIndex]._id,
      this.stakeholder.topics[this.topicIndex].categories[this.categoryIndex].subCategories[this.subcategoryIndex]._id,
      this.section._id
    ];
    this.stakeholderService.saveElement(this.properties.monitorServiceAPIURL, this.indicator, path).subscribe(indicator => {
      if (this.index !== -1) {
        this.section.indicators[this.index] = indicator;
      } else {
        this.section.indicators.push(indicator);
      }
      this.filterCharts();
      this.indicatorFb = null;
    }, error => {
      this.indicatorFb = null;
    });
  }
  
  reorderIndicators(sectionId: string, type: IndicatorType, indicatorIds: string[]) {
    let path = [
      this.stakeholder._id,
      this.stakeholder.topics[this.topicIndex]._id,
      this.stakeholder.topics[this.topicIndex].categories[this.categoryIndex]._id,
      this.stakeholder.topics[this.topicIndex].categories[this.categoryIndex].subCategories[this.subcategoryIndex]._id,
      sectionId
    ];
    this.stakeholderService.reorderIndicators(this.properties.monitorServiceAPIURL, path, indicatorIds, type).subscribe(indicators => {
      if (type === 'chart') {
        this.charts.find(section => section._id === sectionId).indicators = indicators;
        this.filterCharts();
      } else {
        this.numbers.find(section => section._id === sectionId).indicators = indicators;
        this.filterNumbers();
      }
    });
  }
  
  hasDifference(index: number): boolean {
    let hasDifference = false;
    this.indicatorPaths.at(index).value.parameters.forEach((parameter) => {
      if (parameter.value !== this.indicator.indicatorPaths[index].parameters[parameter.key]) {
        hasDifference = true;
        return;
      }
    });
    return hasDifference || this.indicator.indicatorPaths[index].safeResourceUrl.toString() !==
      this.getSecureUrlByStakeHolder(this.indicator.indicatorPaths[index]).toString();
  }
  
  refreshIndicator() {
    this.indicator = this.indicatorUtils.generateIndicatorByForm(this.indicatorFb.value, this.indicator.indicatorPaths);
    this.indicator.indicatorPaths.forEach(indicatorPath => {
      indicatorPath.safeResourceUrl = this.getSecureUrlByStakeHolder(indicatorPath);
    });
  }
  
  deleteIndicatorOpen(section: Section, indicatorId: string, type: string = 'chart') {
    this.section = section;
    this.indicator = section.indicators.find(value => value._id == indicatorId);
    this.deleteModal.alertTitle = 'Delete indicator';
    this.deleteModal.cancelButtonText = 'No';
    this.deleteModal.okButtonText = 'Yes';
    this.deleteModal.open();
  }
  
  deleteIndicator() {
    let path = [
      this.stakeholder._id,
      this.stakeholder.topics[this.topicIndex]._id,
      this.stakeholder.topics[this.topicIndex].categories[this.categoryIndex]._id,
      this.stakeholder.topics[this.topicIndex].categories[this.categoryIndex].subCategories[this.subcategoryIndex]._id,
      this.section._id,
      this.indicator._id
    ];
    this.stakeholderService.deleteElement(this.properties.monitorServiceAPIURL, path).subscribe(() => {
      if (this.indicator.type === 'chart') {
        this.charts.find(section => section._id === this.section._id).indicators.splice(this.index, 1);
      } else {
        this.numbers.find(section => section._id === this.section._id).indicators.splice(this.index, 1);
      }
    });
  }
  
  toggleIndicatorStatus(sectionId: string, indicator: Indicator) {
    let path = [
      this.stakeholder._id,
      this.stakeholder.topics[this.topicIndex]._id,
      this.stakeholder.topics[this.topicIndex].categories[this.categoryIndex]._id,
      this.stakeholder.topics[this.topicIndex].categories[this.categoryIndex].subCategories[this.subcategoryIndex]._id,
      sectionId,
      indicator._id
    ];
    this.stakeholderService.toggleStatus(this.properties.monitorServiceAPIURL, path).subscribe(isActive => {
      indicator.isActive = isActive;
    });
  }
  
  toggleIndicatorAccess(sectionId: string, indicator: Indicator) {
    let path = [
      this.stakeholder._id,
      this.stakeholder.topics[this.topicIndex]._id,
      this.stakeholder.topics[this.topicIndex].categories[this.categoryIndex]._id,
      this.stakeholder.topics[this.topicIndex].categories[this.categoryIndex].subCategories[this.subcategoryIndex]._id,
      sectionId,
      indicator._id
    ];
    this.stakeholderService.toggleAccess(this.properties.monitorServiceAPIURL, path).subscribe(isPublic => {
      indicator.isPublic = isPublic;
    });
  }
}
